import os
import celery


def make_taskrunner(app):
    class AutoTask(celery.Task):
        def __call__(self, *args, **kwargs):
            with app.app_context():
                return self.run(*args, **kwargs)

    celery_app = celery.Celery(
        "tasks",
        task_cls=AutoTask,
        backend=os.environ.get("CELERY_BACKEND", "rpc://user:bitnami@rabbitmq"),
        broker=os.environ.get("CELERY_BROKER_URL", "pyamqp://user:bitnami@rabbitmq"),
    )

    celery_app.config_from_object(app.config["CELERY"])
    celery_app.set_default()
    return celery_app



from autonomous import log
from tasks import mocktask, longmocktask, parametermocktask, errormocktask
import time
from celery.result import AsyncResult
import pytest


def test_base_task(app):
    results = mocktask.delay()
    time.sleep(2)
    log(results.status)
    assert results.status == "SUCCESS"
    assert results.get() == "success"


def test_param_task(app):
    results = parametermocktask.delay(1, 2, "hello", key="value")
    time.sleep(1)
    log(results.status)
    assert results.status == "SUCCESS"
    assert results.get() == 3


def test_error_task(app):
    results = errormocktask.delay()
    time.sleep(2)
    log(results.status)
    assert results.status == "FAILURE"
    try:
        response = results.get()
    except Exception as e:
        log(e)
    else:
        pytest.fail(f"Exception not raised: {response}")


def test_base_long_task(app):
    task = longmocktask.delay()
    result = AsyncResult(task.id)
    log(result.status)
    result.ready()
    assert result.get() == "success"



import time
from celery import shared_task
from autonomous import log
from app import create_app

autotask = create_app().extensions["celery"]


@shared_task()
def mocktask():
    time.sleep(1)
    log("MockTask")
    return "success"


@shared_task()
def longmocktask():
    time.sleep(30)
    return "success"


@shared_task()
def parametermocktask(*args, **kwargs):
    log("ParameterMockTask", args, kwargs)

    return args[0] + args[1]


@shared_task()
def errormocktask():
    raise Exception("ErrorMockTask")



from ..db import db as _database


class ORM:
    def __init__(self, table):
        self._table = _database.get_table(table=table)
        self.name = self._table.name

    @property
    def table(self):
        return self._table

    def save(self, data):
        return self._table.save(data)

    def get(self, pk):
        return self._table.get(pk)

    def all(self):
        return self._table.all()

    def search(self, **kwargs):
        return self._table.search(**kwargs)

    def find(self, **kwargs):
        return self._table.find(**kwargs)

    def delete(self, pk):
        return self._table.delete(pk)

    def flush_table(self):
        return self._table.clear()



from . import Storage
import os
import boto3
from botocore.config import Config
from botocore.exceptions import ClientError
from autonomous import log


class S3Storage(Storage):
    pass
    # def __init__(self):
    #     self.bucket = os.getenv("S3_BUCKET_NAME")
    #     self.path = os.getenv("APP_NAME", "app")
    #     self.client = boto3.client(
    #         "s3",
    #         config=Config(
    #             region_name=os.getenv("AWS_DEFAULT_REGION", "us-east-1"),
    #             signature_version="v4",
    #             retries={"max_attempts": 2, "mode": "standard"},
    #         ),
    #     )

    # def geturl(self, key, **kwargs):
    #     """Generate a presigned URL to share an S3 object
    #     :param key: string
    #     :param expiration: Time in seconds for the presigned URL to remain valid
    #     :return: Presigned URL as string. If error, returns None.
    #     """

    #     # Generate a presigned URL for the S3 object
    #     try:
    #         response = self.client.generate_presigned_url(
    #             "get_object",
    #             Params={"Bucket": self.bucket, "Key": key},
    #             ExpiresIn=kwargs.get("expiration", 3600),
    #         )
    #     except ClientError as e:
    #         log.error(e)
    #         return None

    #     # The response contains the presigned URL
    #     return response



import os


#################################################################
#                         CONFIGURATION                         #
#################################################################
class Config:
    APP_NAME = os.environ.get("app", __name__)
    HOST = os.environ.get("HOST", "0.0.0.0")
    PORT = os.environ.get("PORT", 80)
    SECRET_KEY = os.environ.get("SECRET_KEY", "NATASHA")
    DEBUG = os.environ.get("DEBUG", False)
    TESTING = os.environ.get("TESTING", False)
    TRAP_HTTP_EXCEPTIONS = os.environ.get("TRAP_HTTP_EXCEPTIONS", False)

    CELERY = dict(
        broker_url=os.environ.get("CELERY_BROKER_URL", "pyamqp://user:bitnami@rabbitmq"),
        backend=os.environ.get("CELERY_BACKEND", "rpc://user:bitnami@rabbitmq"),
        results_backend=os.environ.get("CELERY_BACKEND", "rpc://user:bitnami@rabbitmq"),
        task_serializer=os.environ.get("CELERY_SERIALIZER", "json"),
        result_serializer=os.environ.get("CELERY_SERIALIZER", "json"),
        accept_content=os.environ.get(
            "CELERY_ACCEPTED_CONTENT",
            ["json"],
        ),
        track_started=os.environ.get(
            "CELERY_TRACK",
            True,
        ),
        store_errors_even_if_ignored=os.environ.get(
            "CELERY_TRACK",
            True,
        ),
    )



# Built-In Modules

# external Modules
from flask import Blueprint, render_template, request, session
from autonomous import log

admin_page = Blueprint("admin", __name__)


@admin_page.route("/", methods=("GET",))
def index():
    if request.form:
        session.update(request.form)
    return render_template("index.html")


@admin_page.route("/add", methods=("POST",))
def add():
    return {"result": "success"}


@admin_page.route("/update", methods=("POST",))
def updates():
    return "updated"


@admin_page.route("/delete", methods=("POST",))
def delete():
    return "deleted"



# external imports
import os
import pathlib

# from autonomous import log
from .table import Table


class Database:
    def __init__(self, path=""):
        """
        create an interface for your database
        """
        if not path.startswith("/"):
            path = f"{pathlib.Path().resolve()}{path}"
        if not path.endswith("/"):
            path = path + "/"

        self.db_path = f'{path}{os.environ.get("DB_NAME", "tables")}'
        os.path.isdir(self.db_path) or os.makedirs(self.db_path)
        self.tables = {}

    def get_table(self, table="default"):
        """
        opens the table from the file, which clears any changed data
        """
        if not self.tables.get(table):
            self.tables[table] = Table(table, path=self.db_path)
        return self.tables[table]



import os
import slugify


class Storage:
    def __init__(self, path="static"):
        self.path = path

    def geturl(self, key, **kwargs):
        return open(os.path.join(self.path, key))

    def save(self, file, path=None):
        pass

    def remove(self, file):
        pass

    @classmethod
    def getfileid(cls, file):
        filename = os.path.basename(file)
        id = os.path.splitext(filename)[0]
        return slugify(id)



import json

import tinydb

from autonomous import log


class AutoStorage(tinydb.storages.Storage):
    def __init__(self, filename):  # (1)
        self.filename = filename

    def read(self):
        with open(self.filename, "a+") as fptr:
            fptr.seek(0, 0)
            data = fptr.read()
            return data

    def write(self, data):
        with open(self.filename, "w") as fptr:
            serialized_data = json.dumps(data)
            fptr.write(serialized_data)

    def close(self):  # (4)
        pass



import json

import tinydb
from tinydb import where

from autonomous import log

from .storage import AutoStorage


class Table:
    def __init__(self, name, path="."):
        """
        [summary]
        """
        self.path = path
        # breakpoint()
        self._table = tinydb.TinyDB(f"{self.path}/{name}.json").table(name=name)

    @property
    def name(self):
        return self._table.name

    def save(self, obj):
        """
        [summary]
        """
        # log(obj)
        if not obj.get("pk"):
            obj["pk"] = self._table.insert(obj)
        # log(obj, len(self._table))
        self._table.update(obj, doc_ids=[obj["pk"]])
        # log(f"_auto_pk:{pk}")
        return obj["pk"]

    def count(self):
        return len(self._table)

    def delete(self, pk):
        """
        [summary]
        """
        try:
            self._table.remove(
                doc_ids=[
                    pk,
                ]
            )
        except Exception as e:
            log(e)
            return pk
        else:
            return None

    def find(self, **search_terms):
        result = self.search(**search_terms)
        return result[0] if result else None

    def search(self, **search_terms):
        """
        Returns an list of objects based on passed arguments
        as key/value pairs
        """
        matches = []
        for k, v in search_terms.items():
            if v is None:
                continue

            def test_contains(value):
                if isinstance(v, (str)) and isinstance(value, (str)):
                    return v.lower() in value.lower()
                return v == value

            results = self._table.search(where(k).test(test_contains))
            matches += results

        # filtered_matches = []
        # filtered_matches = filter(lambda m: m not in filtered_matches, matches)
        # log(list(matches))
        # log(list(map(lambda m : id(m), matches)))
        return matches

    def get(self, pk):
        """
        [summary]
        """

        obj = None
        if isinstance(pk, int):
            # breakpoint()
            obj = self._table.get(doc_id=pk)
            # breakpoint()
            # log(obj_id, obj, self, self._table)
        return obj

    def all(self):
        result = self._table.all()
        # log(result)
        return result

    def __str__(self):
        return json.dumps(self.all(), indent=4)

    def clear(self):
        len(self._table) and self._table.truncate()
